#pragma once
#include <glew.h>
#include <glut.h>
#include <string>
#include <fstream>
#include <vector>
#include <memory>
#include <iostream>
#include <sstream>
#include <streambuf>  
using namespace std;
namespace OpenglTest
{
	class glslProgram
	{
	public:
		glslProgram();
		~glslProgram();
		inline void enable(){ glUseProgram(programId); };
		inline void disable(){ glUseProgram(0); };

		inline unique_ptr<string> readFile(const string fileName) const
		{
			fstream file(fileName, std::ios::in | std::ios::binary);
			if (!file.is_open())
				throw std::invalid_argument("Error: File Not Found.");

			//stringstream buffer;
			//buffer << file.rdbuf();
			//string* content = new string(buffer.str());
			//return unique_ptr<string>(content);

			std::string str((istreambuf_iterator<char>(file)),
				std::istreambuf_iterator<char>());
			return unique_ptr<string>(new string(str));
		}

		inline void initShader(const string vertFile, const string fragFile)
		{
			programId = glCreateProgram();
			vertId = glCreateShader(GL_VERTEX_SHADER);
			fragId = glCreateShader(GL_FRAGMENT_SHADER);

			auto vertSource = readFile(vertFile);
			auto fragSource = readFile(fragFile);
			auto vs = vertSource->data();
			auto fs = fragSource->data();

			glShaderSource(vertId, 1, &vs, nullptr);
			glCompileShader(vertId);
			printShaderInfoLog(vertId);

			glShaderSource(fragId, 1, &fs, nullptr);
			glCompileShader(fragId);
			printShaderInfoLog(fragId);

			glAttachShader(programId, vertId);
			glAttachShader(programId, fragId);
		}

		inline void linkShader()
		{
			glLinkProgram(programId);
			printProgramInfoLog(programId);
			validateShader();
		}

		inline void setUniform(const string& uniformName, float x)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform1f(id, x);
		}
		inline void setUniform(const string& uniformName, float x, float y)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform2f(id, x, y);
		}
		inline void setUniform(const string& uniformName, float x, float y, float z)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform3f(id, x, y, z);
		}
		inline void setUniform(const string& uniformName, float x, float y, float z, float w)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform4i(id, x, y, z, w);
		}
		inline void setUniform(const string& uniformName, int x)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform1i(id, x);
		}
		inline void setUniform(const string& uniformName, int x, int y)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform2i(id, x, y);
		}
		inline void setUniform(const string& uniformName, int x, int y, int z)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform3i(id, x, y, z);
		}
		inline void setUniform(const string& uniformName, int x, int y, int z, int w)
		{
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniform4i(id, x, y, z, w);
		}

		inline void setUniformMatrix(const string& uniformName, const float* value)
		{
			//glUniformMatrix4fv(GLint location,  GLsizei count,  GLboolean transpose,  const GLfloat *value)
			auto id = glGetUniformLocation(programId, uniformName.data());
			glUniformMatrix4fv(id, 1, false, value);
			//glUniform4i(id, x, y, z, w);
		}

		inline void setAttributeLocation(const string& AttributeName, GLuint location)
		{
			glBindAttribLocation(programId, location, AttributeName.data());
		}

		inline void printShaderInfoLog(GLuint shaderID)
		{
			int logLength;
			glGetShaderiv(shaderID, GL_INFO_LOG_LENGTH, &logLength);

			if (logLength > 1)
			{
				vector<char> logs;
				logs.resize(logLength);
				glGetShaderInfoLog(shaderID, logLength, nullptr, logs.data());
				string logInfo(logs.data());
				cout << logInfo << endl;
			}
		}

		inline void printProgramInfoLog(GLuint programID)
		{
			int logLength;
			glGetProgramiv(programID, GL_INFO_LOG_LENGTH, &logLength);
			if (logLength > 1)
			{
				vector<char> logs;
				logs.resize(logLength);
				glGetProgramInfoLog(programID, logLength, nullptr, logs.data());
				string logInfo(logs.data());
				cout << logInfo << endl;
			}
		}

		inline void validateShader()
		{
			glValidateProgram(programId);
			GLint status;
			glGetProgramiv(programId, GL_VALIDATE_STATUS, &status);
			if (!status)
			{
				throw exception("glsl code is error. please look at the output of the error message");
			}
		}

	private:
		GLuint programId;
		GLuint vertId;
		GLuint fragId;
	};


}

